//
// Copyright (c) 2008 smdn <smdn@smdn.jp>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Text;

using Smdn.IO.Streams;
using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.DataStructures;
using Smdn.Net.Imap4.Protocol.DataStructures.MessageAttributes;
using Smdn.Net.Imap4.Protocol.Transmission;
using Smdn.Net.MessageAccessProtocols.Strings;

namespace Smdn.Net.Imap4.Client {
  partial class ImapMailbox {
    /*
     * APPEND
     */
    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public void AppendMessage(Stream stream,
                              IEnumerable<ImapMessageFlag> flags)
      => AppendMessage(stream, appendDate: null, flags: flags);

    public void AppendMessage(Stream stream,
                              DateTimeOffset? appendDate = null,
                              IEnumerable<ImapMessageFlag> flags = null)
      => AppendMessage(new ImapAppendMessage(stream ?? throw new ArgumentNullException(nameof(stream)),
                                             stream.Length,
                                             appendDate,
                                             flags));

    public void AppendMessage(IImapAppendMessage message)
    {
      if (message == null)
        throw new ArgumentNullException(nameof(message));

      ThrowIfNonExistentOrUnselectable();

      client.RunExclusively(() => {
        ProcessResult(Client.Session.Append(message, Mailbox, out _),
                      ThrowIfAppendErrorResponse);
      });
    }

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task AppendMessageAsync(Stream stream,
                                   IEnumerable<ImapMessageFlag> flags)
      => AppendMessageAsync(stream, appendDate: null, flags: flags);

    public Task AppendMessageAsync(Stream stream,
                                   DateTimeOffset? appendDate = null,
                                   IEnumerable<ImapMessageFlag> flags = null)
      => AppendMessageAsync(new ImapAppendMessage(stream ?? throw new ArgumentNullException(nameof(stream)),
                                                  stream.Length,
                                                  appendDate,
                                                  flags));

    public Task AppendMessageAsync(IImapAppendMessage message)
    {
      if (message == null)
        throw new ArgumentNullException(nameof(message));

      ThrowIfNonExistentOrUnselectable();

      return client.StartTaskExclusively(() => {
        ProcessResult(Client.Session.Append(message, Mailbox, out _),
                      ThrowIfAppendErrorResponse);
      }, CancellationToken.None);
    }

    public void AppendMessages(IEnumerable<IImapAppendMessage> messages)
    {
      if (messages == null)
        throw new ArgumentNullException(nameof(messages));

      ThrowIfNonExistentOrUnselectable();

      client.RunExclusively(() => AppendMessagesCore(messages));
    }

    public Task AppendMessagesAsync(IEnumerable<IImapAppendMessage> messages)
    {
      if (messages == null)
        throw new ArgumentNullException(nameof(messages));

      ThrowIfNonExistentOrUnselectable();

      return client.StartTaskExclusively(() => AppendMessagesCore(messages),
                                         CancellationToken.None);
    }

    private void AppendMessagesCore(IEnumerable<IImapAppendMessage> messages)
    {
      if (Client.IsCapabilityAvailable(ImapCapability.MultiAppend)) {
        ProcessResult(Client.Session.Append(messages, Mailbox, out _),
                      ThrowIfAppendErrorResponse);
      }
      else {
        foreach (var message in messages) {
          ProcessResult(Client.Session.Append(message, Mailbox, out _),
                        ThrowIfAppendErrorResponse);
        }
      }
    }

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public void AppendMessage(DateTimeOffset appendDate,
                              Action<Stream> writeMessage)
      => AppendMessage(writeMessage, appendDate: appendDate, flags: null);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public void AppendMessage(IEnumerable<ImapMessageFlag> flags,
                              Action<Stream> writeMessage)
      => AppendMessage(writeMessage, appendDate: null, flags: flags);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public void AppendMessage(DateTimeOffset appendDate,
                              IEnumerable<ImapMessageFlag> flags,
                              Action<Stream> writeMessage)
      => AppendMessage(writeMessage, appendDate: appendDate, flags: flags);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public void AppendMessage(long length,
                              DateTimeOffset appendDate,
                              Action<Stream> writeMessage)
      => AppendMessage(length, writeMessage, appendDate: appendDate, flags: null);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public void AppendMessage(long length,
                              IEnumerable<ImapMessageFlag> flags,
                              Action<Stream> writeMessage)
      => AppendMessage(length, writeMessage, appendDate: null, flags: flags);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public void AppendMessage(long length,
                              DateTimeOffset appendDate,
                              IEnumerable<ImapMessageFlag> flags,
                              Action<Stream> writeMessage)
      => AppendMessage(length, writeMessage, appendDate: appendDate, flags: flags);

    public void AppendMessage(Action<Stream> writeMessage,
                              DateTimeOffset? appendDate = null,
                              IEnumerable<ImapMessageFlag> flags = null)
      => AppendMessageWithWriteAction(null,
                                      appendDate,
                                      flags,
                                      writeMessage);

    public void AppendMessage(long length,
                              Action<Stream> writeMessage,
                              DateTimeOffset? appendDate = null,
                              IEnumerable<ImapMessageFlag> flags = null)
      => AppendMessageWithWriteAction(length,
                                      appendDate,
                                      flags,
                                      writeMessage);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task AppendMessageAsync(DateTimeOffset appendDate,
                                   Action<Stream> writeMessage)
      => AppendMessageAsync(writeMessage, appendDate: appendDate, flags: null);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task AppendMessageAsync(IEnumerable<ImapMessageFlag> flags,
                                   Action<Stream> writeMessage)
      => AppendMessageAsync(writeMessage, appendDate: null, flags: flags);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task AppendMessageAsync(DateTimeOffset appendDate,
                                   IEnumerable<ImapMessageFlag> flags,
                                   Action<Stream> writeMessage)
      => AppendMessageAsync(writeMessage, appendDate: appendDate, flags: flags);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task AppendMessageAsync(long length,
                                   DateTimeOffset appendDate,
                                   Action<Stream> writeMessage)
      => AppendMessageAsync(length, writeMessage, appendDate: appendDate, flags: null);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task AppendMessageAsync(long length,
                                   IEnumerable<ImapMessageFlag> flags,
                                   Action<Stream> writeMessage)
      => AppendMessageAsync(length, writeMessage, appendDate: null, flags: flags);

    [Obsolete(ImapClient.ObsoleteAttributeMessage_UseOptionalParameterOverloads)]
    public Task AppendMessageAsync(long length,
                                   DateTimeOffset appendDate,
                                   IEnumerable<ImapMessageFlag> flags,
                                   Action<Stream> writeMessage)
      => AppendMessageAsync(length, writeMessage, appendDate: appendDate, flags: flags);

    public Task AppendMessageAsync(Action<Stream> writeMessage,
                                   DateTimeOffset? appendDate = null,
                                   IEnumerable<ImapMessageFlag> flags = null)
      => AppendMessageWithWriteActionAsync(null,
                                           appendDate,
                                           flags,
                                           writeMessage);

    public Task AppendMessageAsync(long length,
                                   Action<Stream> writeMessage,
                                   DateTimeOffset? appendDate = null,
                                   IEnumerable<ImapMessageFlag> flags = null)
      => AppendMessageWithWriteActionAsync(length,
                                           appendDate,
                                           flags,
                                           writeMessage);

    private void AppendMessageWithWriteAction(long? length,
                                              DateTimeOffset? appendDate,
                                              IEnumerable<ImapMessageFlag> flags,
                                              Action<Stream> writeMessage)
    {
      if (writeMessage == null)
        throw new ArgumentNullException(nameof(writeMessage));
      if (length.HasValue && length.Value < 0L)
        throw ExceptionUtils.CreateArgumentMustBeZeroOrPositive(nameof(length), length.Value);

      ThrowIfNonExistentOrUnselectable();

      client.RunExclusively(() => AppendMessageCore(length,
                                                    appendDate,
                                                    flags,
                                                    writeMessage));
    }

    private Task AppendMessageWithWriteActionAsync(long? length,
                                                   DateTimeOffset? appendDate,
                                                   IEnumerable<ImapMessageFlag> flags,
                                                   Action<Stream> writeMessage)
    {
      if (writeMessage == null)
        throw new ArgumentNullException(nameof(writeMessage));
      if (length.HasValue && length.Value < 0L)
        throw ExceptionUtils.CreateArgumentMustBeZeroOrPositive(nameof(length), length.Value);

      ThrowIfNonExistentOrUnselectable();

      return client.StartTaskExclusively(() => AppendMessageCore(length,
                                                                 appendDate,
                                                                 flags,
                                                                 writeMessage),
                                         CancellationToken.None);
    }

    public void AppendMessage(ImapMessage message)
    {
      if (message == null)
        throw new ArgumentNullException(nameof(message));

      if (message.Client == this.client) {
        message.CopyTo(this);
      }
      else {
        ThrowIfNonExistentOrUnselectable();

        client.RunExclusively(() => CopyMessageAcrossSessionCore(message, true));
      }
    }

    public Task AppendMessageAsync(ImapMessage message)
    {
      if (message == null)
        throw new ArgumentNullException(nameof(message));

      if (message.Client == this.client) {
        return message.CopyToAsync(this);
      }
      else {
        ThrowIfNonExistentOrUnselectable();

        return client.StartTaskExclusively(() => CopyMessageAcrossSessionCore(message, true),
                                           CancellationToken.None);
      }
    }

    private static readonly ImapFetchDataItems appendingDataItems =
      ImapFetchDataItems.Rfc822Size +
      ImapFetchDataItems.InternalDate +
      ImapFetchDataItems.Flags;

    internal void CopyMessageAcrossSession(ImapMessage message, bool needToSetMessageBusy)
    {
#if DEBUG
      if (message.Client == this.client)
        throw new NotSupportedException("use ImapMessage.CopyTo");
#endif

      client.RunExclusively(() => CopyMessageAcrossSessionCore(message, needToSetMessageBusy));
    }

    private void CopyMessageAcrossSessionCore(ImapMessage message, bool needToSetMessageBusy)
    {
      message.Body.ReadMessageBody(
        setBusy: needToSetMessageBusy,
        messageStreamConverter: _CopyMessage,
        messageStreamConverterArgument: default(int),
        options: ImapMessageFetchBodyOptions.Default,
        fetchDataItems: appendingDataItems,
        cancellationToken: default,
        progress: default
      );

      int _CopyMessage(Stream sourceStream, Encoding _0, ImapMessageAttributes attrs, int _1)
      {
        AppendMessageCore(attrs.Rfc822Size,
                          attrs.InternalDate,
                          attrs.Flags,
                          destinationStream => ImapMessageBody.CopyFetchMessageBodyStream(sourceStream, destinationStream));

        return default;
      }
    }

    private class FromActionAppendMessage : IImapAppendMessage, IDisposable {
      public long Length => EnsureMessageStreamWritten().Length;
      public DateTimeOffset? InternalDate { get; private set; }
      public IReadOnlyStringSet<ImapMessageFlag> Flags { get; private set; }

      public FromActionAppendMessage(long? length,
                                     DateTimeOffset? appendDate,
                                     IEnumerable<ImapMessageFlag> flags,
                                     Action<Stream> writeMessage)
      {
        this.length = length;
        this.InternalDate = appendDate;
        this.Flags = (flags == null) ? null : ImapMessageFlagSet.ToSet(flags);
        this.writeMessage = writeMessage;
      }

      public void Dispose()
      {
        messageStream?.Dispose();
        messageStream = null;

#if DEBUG
        if (bufferStream.CanWrite)
          throw new Exception("buffer not disposed");
#endif
      }

      public Stream GetMessageStream()
      {
        return EnsureMessageStreamWritten();
      }

      private Stream EnsureMessageStreamWritten()
      {
        if (messageStream == null) {
          var buffer = new ChunkedMemoryStream();
#if DEBUG
          bufferStream = buffer;
#endif

          using (var stream = new NonClosingStream(buffer)) {
            writeMessage(stream);
          }

          buffer.Position = 0L;

          if (length.HasValue) {
            if (buffer.Length < length.Value) {
              buffer.Dispose();
              throw new InvalidOperationException(string.Format("specified length is {0}, but written message length was {1}", length, buffer.Length));
            }

            messageStream = new PartialStream(buffer, 0L, length.Value, @readonly: true, leaveInnerStreamOpen: false);
          }
          else {
            messageStream = buffer;
          }
        }

        return messageStream;
      }

      private readonly long? length;
      private readonly Action<Stream> writeMessage;
      private Stream messageStream;
#if DEBUG
      private ChunkedMemoryStream bufferStream;
#endif
    }

    private void AppendMessageCore(long? length,
                                   DateTimeOffset? appendDate,
                                   IEnumerable<ImapMessageFlag> flags,
                                   Action<Stream> writeMessage)
    {
      using (var message = new FromActionAppendMessage(length, appendDate, flags, writeMessage)) {
        ProcessResult(client.Session.Append(message, mailbox, out _),
                      ThrowIfAppendErrorResponse);
      }
    }

    private bool ThrowIfAppendErrorResponse(ImapTaggedStatusResponse status)
    {
      if (status.ResponseText.Code == ImapResponseCode.TryCreate)
        throw new ImapMailboxNotFoundException(FullName);
      else
        return true;
    }
  }
}

