//
// Copyright (c) 2008 smdn <smdn@smdn.jp>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Threading;

using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.DataStructures;
using Smdn.Net.Imap4.Protocol.DataStructures.MessageAttributes;
using Smdn.Net.Imap4.Protocol.Session;
using Smdn.Net.Imap4.Protocol.Transmission;

namespace Smdn.Net.Imap4.Client {
  partial class ImapMessageBodyBase {
    private sealed class FetchMessageBodyStream : ImapFetchMessageBodyStream {
      public ImapMessage Message { get; private set; }

      public override long Length => throw ExceptionUtils.CreateNotSupportedSeekingStream();

      private readonly bool unsetBusy;
      private readonly IProgress<ImapMessageFetchProgressInfo> progress;

      public FetchMessageBodyStream(ImapMessage message,
                                    bool peek,
                                    string sectionSpecifier,
                                    ImapFetchRange range,
                                    CancellationToken cancellationToken,
                                    IProgress<ImapMessageFetchProgressInfo> progress,
                                    bool unsetBusy = true)
        : base(message.Client.Session,
               uid: ImapUidSet.Of(message.Uid),
               peek: peek,
               sectionSpecifier: sectionSpecifier,
               range: range,
               cancellationToken: cancellationToken)
      {
        this.Message = message;
        this.unsetBusy = unsetBusy;
        this.progress = progress;
      }

      protected override void Dispose(bool disposing)
      {
        if (Message != null) {
          if (unsetBusy)
            Message.Client.UnsetBusy();

          Message = null;
        }

        base.Dispose(disposing);
      }

      internal protected override (ImapCommandResult, ImapMessageAttributes) Prepare(ImapFetchDataItems fetchDataItems)
      {
        CheckDisposed();

        try {
          var (result, messageAttributes) = base.Prepare(fetchDataItems);

          return (Message.Mailbox.ProcessResult(result), messageAttributes);
        }
        catch {
          Dispose();

          throw;
        }
      }

      internal protected override void ReportFetchedOctetCount(long fetchedOctetCount, long totalOctetCount)
        => progress?.Report(new ImapMessageFetchProgressInfo(fetchedOctetCount, totalOctetCount, CancellationToken));

      protected override Exception CreateNoSuchMessageException(ImapCommandResult result)
      {
        CheckDisposed();

        Message.Mailbox.ProcessResult(result);

        Message.SetDeleted();

        return new ImapMessageVanishedException(Message);
      }

      protected override Exception CreateFetchFailureException(ImapCommandResult result)
      {
        CheckDisposed();

        Message.Mailbox.ProcessResult(result);

        return base.CreateFetchFailureException(result);
      }

      private void CheckDisposed()
      {
        if (Message == null)
          throw new ObjectDisposedException(GetType().FullName);
      }
    }
  }
}

