// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
#if SERIALIZATION
using System.Runtime.Serialization;
#endif

using Smdn.Net.Imap4.Protocol.DataStructures;

namespace Smdn.Net.Imap4.Client {
  // Exceptions:
  //   System.Excpetion
  //     System.SystemException
  //       Smdn.Net.Imap4.ImapExcetion
  //         Smdn.Net.Imap4.Client.ImapMessageNotFoundException

#if SERIALIZATION
  [Serializable]
#endif
  public class ImapMessageNotFoundException : ImapException {
    private readonly IImapNumberSet sequenceOrUidSet;

    public IImapNumberSet SequenceOrUidSet {
      get { return sequenceOrUidSet; }
    }

    public ImapMessageNotFoundException(IImapNumberSet sequenceOrUidSet)
      : this($"no such message: {sequenceOrUidSet}", sequenceOrUidSet)
    {
    }

    public ImapMessageNotFoundException(string message, IImapNumberSet sequenceOrUidSet)
      : base(message)
    {
      if (sequenceOrUidSet == null)
        throw new ArgumentNullException(nameof(sequenceOrUidSet));

      this.sequenceOrUidSet = sequenceOrUidSet;
    }

#if SERIALIZATION
    protected ImapMessageNotFoundException(SerializationInfo info, StreamingContext context)
      : base(info, context)
    {
      this.sequenceOrUidSet = (IImapNumberSet)info.GetValue("SequenceOrUidSet", typeof(IImapNumberSet));
    }

    public override void GetObjectData(SerializationInfo info, StreamingContext context)
    {
      base.GetObjectData(info, context);

      info.AddValue("SequenceOrUidSet", sequenceOrUidSet);
    }
#endif
  }
}
