//
// Copyright (c) 2008 smdn <smdn@smdn.jp>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

using Smdn.Net.Imap4.Protocol.DataStructures;
using Smdn.Text.Encodings;

namespace Smdn.Net.Imap4.Client {
  partial class ImapMessagePart {
    /*
     * OpenRead()
     */
    [Obsolete("use Body.Read(Action<Stream>) or Body.Download() instead")]
    public Stream OpenRead()
      => Body.Download();

    [Obsolete("use Body.Read(Action<Stream>) or Body.Download() instead")]
    public Stream OpenRead(ImapMessageFetchBodyOptions options)
      => Body.Download(options: options);

    [Obsolete("use Body.Read(Action<Stream>) or Body.Download() instead")]
    public Stream OpenRead(ImapFetchRange? fetchRange)
      => Body.Download(fetchRange: fetchRange ?? ImapFetchRange.All);

    [Obsolete("use Body.Read(Action<Stream>) or Body.Download() instead")]
    public Stream OpenRead(ImapFetchRange? fetchRange,
                           ImapMessageFetchBodyOptions options)
      => Body.Download(options: options, fetchRange: fetchRange ?? ImapFetchRange.All);

    /*
     * ReadAs<TResult>(Converter<Stream>)
     */
    [Obsolete("use Body.ReadAs() instead")]
    public TResult ReadAs<TResult>(Converter<Stream, TResult> converter)
      => Body.ReadAs(converter);

    [Obsolete("use Body.ReadAs() instead")]
    public TResult ReadAs<TResult>(ImapMessageFetchBodyOptions options,
                                   Converter<Stream, TResult> converter)
      => Body.ReadAs(converter, options: options);

    /*
     * ReadAllBytes()
     */
    [Obsolete("use Body.ReadAsByteArray() instead")]
    public byte[] ReadAllBytes()
      => Body.ReadAsByteArray();

    [Obsolete("use Body.ReadAsByteArray() instead")]
    public byte[] ReadAllBytes(ImapMessageFetchBodyOptions options)
      => Body.ReadAsByteArray(options: options);

    /*
     * WriteTo(Stream)
     */
    [Obsolete("use Body.WriteTo() instead")]
    public void WriteTo(Stream stream)
      => Body.WriteTo(stream);

    [Obsolete("use Body.WriteTo() instead")]
    public void WriteTo(Stream stream,
                        ImapMessageFetchBodyOptions options)
      => Body.WriteTo(stream, options: options);

    /*
     * WriteTo(BinaryWriter)
     */
    [Obsolete("use Body.WriteTo() instead")]
    public void WriteTo(BinaryWriter writer)
      => Body.WriteTo(writer);

    [Obsolete("use Body.WriteTo() instead")]
    public void WriteTo(BinaryWriter writer,
                        ImapMessageFetchBodyOptions options)
      => Body.WriteTo(writer, options: options);

    /*
     * Save()
     */
    [Obsolete("use Body.Save() instead")]
    public void Save(string path)
      => Body.Save(path);

    [Obsolete("use Body.Save() instead")]
    public void Save(string path,
                     ImapMessageFetchBodyOptions options)
      => Body.Save(path, options: options);

    /*
     * Download()
     */
    [Obsolete("use Body.Download() instead")]
    public Stream Download()
      => Body.Download();

    [Obsolete("use Body.Download() instead")]
    public Stream Download(ImapMessageFetchBodyOptions options)
      => Body.Download(options: options);

    /*
     * OpenText
     */
    [Obsolete("use Body.Read(Action<StreamReader>) or Body.ReadAsString() instead")]
    public StreamReader OpenText()
      => Body.ReadAs(ImapMessage.OpenTextConverter, arg: default(int));

    [Obsolete("use Body.Read(Action<StreamReader>) or Body.ReadAsString() instead")]
    public StreamReader OpenText(ImapMessageFetchBodyOptions options)
      => Body.ReadAs(ImapMessage.OpenTextConverter, arg: default(int), options: options);

    [Obsolete("use Body.Read(Action<StreamReader>) or Body.ReadAsString() instead")]
    public StreamReader OpenText(ImapMessageFetchBodyOptions options,
                                 EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAs(ImapMessage.OpenTextConverter, arg: default(int), options: options, selectFallbackCharset: selectFallbackCharset);

    /*
     * ReadAs<TResult>(Converter<StreamReader>)
     */
    [Obsolete("use Body.ReadAs() instead")]
    public TResult ReadAs<TResult>(Converter<StreamReader, TResult> converter)
      => Body.ReadAs(converter);

    [Obsolete("use Body.ReadAs() instead")]
    public TResult ReadAs<TResult>(ImapMessageFetchBodyOptions options,
                                   Converter<StreamReader, TResult> converter)
      => Body.ReadAs(converter, options: options);

    [Obsolete("use Body.ReadAs() instead")]
    public TResult ReadAs<TResult>(ImapMessageFetchBodyOptions options,
                                   EncodingSelectionCallback selectFallbackCharset,
                                   Converter<StreamReader, TResult> converter)
      => Body.ReadAs(converter, options: options, selectFallbackCharset: selectFallbackCharset);

    /*
     * ReadLines()
     */
    [Obsolete("use Body.ReadAsStringLines() instead")]
    public IEnumerable<string> ReadLines()
      => Body.ReadAsStringLines();

    [Obsolete("use Body.ReadAsStringLines() instead")]
    public IEnumerable<string> ReadLines(ImapMessageFetchBodyOptions options)
      => Body.ReadAsStringLines(options: options);

    [Obsolete("use Body.ReadAsStringLines() instead")]
    public IEnumerable<string> ReadLines(ImapMessageFetchBodyOptions options,
                                         EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsStringLines(options: options, selectFallbackCharset: selectFallbackCharset);

    /*
     * ReadAllLines()
     */
    [Obsolete("use Body.ReadAsStringLines() instead")]
    public string[] ReadAllLines()
      => Body.ReadAsStringLines().ToArray();

    [Obsolete("use Body.ReadAsStringLines() instead")]
    public string[] ReadAllLines(ImapMessageFetchBodyOptions options)
      => Body.ReadAsStringLines(options: options).ToArray();

    [Obsolete("use Body.ReadAsStringLines() instead")]
    public string[] ReadAllLines(ImapMessageFetchBodyOptions options,
                                 EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsStringLines(options: options, selectFallbackCharset: selectFallbackCharset).ToArray();

    /*
     * ReadAllText()
     */
    [Obsolete("use Body.ReadAsString() instead")]
    public string ReadAllText()
      => Body.ReadAsString();

    [Obsolete("use Body.ReadAsString() instead")]
    public string ReadAllText(ImapMessageFetchBodyOptions options)
      => Body.ReadAsString(options: options);

    [Obsolete("use Body.ReadAsString() instead")]
    public string ReadAllText(ImapMessageFetchBodyOptions options,
                              EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsString(options: options, selectFallbackCharset: selectFallbackCharset);

    /*
     * OpenReadAsync()
     */
    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync()
      => Body.DownloadAsync();

    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync(CancellationToken cancellationToken)
      => Body.DownloadAsync(cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync(IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(progress: progress);

    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync(CancellationToken cancellationToken,
                                      IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync(ImapMessageFetchBodyOptions options)
      => Body.DownloadAsync(options: options);

    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync(ImapMessageFetchBodyOptions options,
                                      CancellationToken cancellationToken)
      => Body.DownloadAsync(options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync(ImapMessageFetchBodyOptions options,
                                      IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(options: options, progress: progress);

    [Obsolete("use Body.ReadAsync(Action<Stream>) or Body.DownloadAsync() instead")]
    public Task<Stream> OpenReadAsync(ImapMessageFetchBodyOptions options,
                                      CancellationToken cancellationToken,
                                      IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(options: options, cancellationToken: cancellationToken, progress: progress);

    /*
     * ReadAsAsync<TResult>(Converter<Stream>)
     */
    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<Stream, TResult> converter)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)));

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<Stream, TResult> converter,
                                              CancellationToken cancellationToken)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<Stream, TResult> converter,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), progress: progress);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<Stream, TResult> converter,
                                              CancellationToken cancellationToken,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<Stream, TResult> converter)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<Stream, TResult> converter,
                                              CancellationToken cancellationToken)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<Stream, TResult> converter,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, progress: progress);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<Stream, TResult> converter,
                                              CancellationToken cancellationToken,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, cancellationToken: cancellationToken, progress: progress);

    /*
     * ReadAllBytesAsync()
     */
    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync()
      => Body.ReadAsByteArrayAsync();

    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync(CancellationToken cancellationToken)
      => Body.ReadAsByteArrayAsync(cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync(IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsByteArrayAsync(progress: progress);

    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync(CancellationToken cancellationToken,
                                          IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsByteArrayAsync(cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync(ImapMessageFetchBodyOptions options)
      => Body.ReadAsByteArrayAsync(options: options);

    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync(ImapMessageFetchBodyOptions options,
                                          CancellationToken cancellationToken)
      => Body.ReadAsByteArrayAsync(options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync(ImapMessageFetchBodyOptions options,
                                          IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsByteArrayAsync(options: options, progress: progress);

    [Obsolete("use Body.ReadAsByteArrayAsync() instead")]
    public Task<byte[]> ReadAllBytesAsync(ImapMessageFetchBodyOptions options,
                                          CancellationToken cancellationToken,
                                          IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsByteArrayAsync(options: options, cancellationToken: cancellationToken, progress: progress);

    /*
     * WriteToAsync(Stream)
     */
    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream)
      => Body.WriteToAsync(stream);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream,
                             CancellationToken cancellationToken)
      => Body.WriteToAsync(stream, cancellationToken: cancellationToken);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(stream, progress: progress);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream,
                             CancellationToken cancellationToken,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(stream, cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream,
                             ImapMessageFetchBodyOptions options)
      => Body.WriteToAsync(stream, options: options);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream,
                             ImapMessageFetchBodyOptions options,
                             CancellationToken cancellationToken)
      => Body.WriteToAsync(stream, options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream,
                             ImapMessageFetchBodyOptions options,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(stream, options: options, progress: progress);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(Stream stream,
                             ImapMessageFetchBodyOptions options,
                             CancellationToken cancellationToken,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(stream, options: options, cancellationToken: cancellationToken, progress: progress);

    /*
     * WriteToAsync(BinaryWriter)
     */
    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer)
      => Body.WriteToAsync(writer);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer,
                             CancellationToken cancellationToken)
      => Body.WriteToAsync(writer, cancellationToken: cancellationToken);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(writer, progress: progress);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer,
                             CancellationToken cancellationToken,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(writer, cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer,
                             ImapMessageFetchBodyOptions options)
      => Body.WriteToAsync(writer, options: options);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer,
                             ImapMessageFetchBodyOptions options,
                             CancellationToken cancellationToken)
      => Body.WriteToAsync(writer, options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer,
                             ImapMessageFetchBodyOptions options,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(writer, options: options, progress: progress);

    [Obsolete("use Body.WriteToAsync() instead")]
    public Task WriteToAsync(BinaryWriter writer,
                             ImapMessageFetchBodyOptions options,
                             CancellationToken cancellationToken,
                             IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.WriteToAsync(writer, options: options, cancellationToken: cancellationToken, progress: progress);

    /*
     * SaveAsync()
     */
    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path)
      => Body.SaveAsync(path);

    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path,
                          CancellationToken cancellationToken)
      => Body.SaveAsync(path, cancellationToken: cancellationToken);

    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path,
                          IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.SaveAsync(path, progress: progress);

    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path,
                          CancellationToken cancellationToken,
                          IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.SaveAsync(path, cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path,
                          ImapMessageFetchBodyOptions options)
      => Body.SaveAsync(path, options: options);

    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path,
                          ImapMessageFetchBodyOptions options,
                          CancellationToken cancellationToken)
      => Body.SaveAsync(path, options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path,
                          ImapMessageFetchBodyOptions options,
                          IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.SaveAsync(path, options: options, progress: progress);

    [Obsolete("use Body.SaveAsync() instead")]
    public Task SaveAsync(string path,
                          ImapMessageFetchBodyOptions options,
                          CancellationToken cancellationToken,
                          IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.SaveAsync(path, options: options, cancellationToken: cancellationToken, progress: progress);

    /*
     * DownloadAsync()
     */
    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync()
      => Body.DownloadAsync();

    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync(CancellationToken cancellationToken)
      => Body.DownloadAsync(cancellationToken: cancellationToken);

    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync(IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(progress: progress);

    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync(CancellationToken cancellationToken,
                                      IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync(ImapMessageFetchBodyOptions options)
      => Body.DownloadAsync(options: options);

    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync(ImapMessageFetchBodyOptions options,
                                      CancellationToken cancellationToken)
      => Body.DownloadAsync(options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync(ImapMessageFetchBodyOptions options,
                                      IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(options: options, progress: progress);

    [Obsolete("use Body.DownloadAsync() instead")]
    public Task<Stream> DownloadAsync(ImapMessageFetchBodyOptions options,
                                      CancellationToken cancellationToken,
                                      IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.DownloadAsync(options: options, cancellationToken: cancellationToken, progress: progress);

    /*
     * OpenTextAsync()
     */
    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync()
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int));

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(CancellationToken cancellationToken)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), progress: progress);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(CancellationToken cancellationToken,
                                            IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(ImapMessageFetchBodyOptions options)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), options: options);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(ImapMessageFetchBodyOptions options,
                                            CancellationToken cancellationToken)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(ImapMessageFetchBodyOptions options,
                                            EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), options: options, selectFallbackCharset: selectFallbackCharset);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(ImapMessageFetchBodyOptions options,
                                            IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), options: options, progress: progress);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(ImapMessageFetchBodyOptions options,
                                            CancellationToken cancellationToken,
                                            IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), options: options, cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsync(Action<StreamReader>) or Body.ReadAsStringAsync() instead")]
    public Task<StreamReader> OpenTextAsync(ImapMessageFetchBodyOptions options,
                                            CancellationToken cancellationToken,
                                            IProgress<ImapMessageFetchProgressInfo> progress,
                                            EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsAsync(ImapMessage.OpenTextAsyncConverter, arg: default(int), options: options, cancellationToken: cancellationToken, progress: progress, selectFallbackCharset: selectFallbackCharset);

    /*
     * ReadAsAsync<TResult>(Converter<StreamReader>)
     */
    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<StreamReader, TResult> converter)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)));

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<StreamReader, TResult> converter,
                                              CancellationToken cancellationToken)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<StreamReader, TResult> converter,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), progress: progress);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(Converter<StreamReader, TResult> converter,
                                              CancellationToken cancellationToken,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<StreamReader, TResult> converter)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<StreamReader, TResult> converter,
                                              CancellationToken cancellationToken)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<StreamReader, TResult> converter,
                                              EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, selectFallbackCharset: selectFallbackCharset);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<StreamReader, TResult> converter,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, progress: progress);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              Converter<StreamReader, TResult> converter,
                                              CancellationToken cancellationToken,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsAsync() instead")]
    public Task<TResult> ReadAsAsync<TResult>(ImapMessageFetchBodyOptions options,
                                              EncodingSelectionCallback selectFallbackCharset,
                                              Converter<StreamReader, TResult> converter,
                                              CancellationToken cancellationToken,
                                              IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsAsync(converter: ImapMessage.ReadAsAsyncConverter, arg: converter ?? throw new ArgumentNullException(nameof(converter)), options: options, selectFallbackCharset: selectFallbackCharset, cancellationToken: cancellationToken, progress: progress);

    /*
     * ReadAllLinesAsync()
     */
    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync()
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync());

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(CancellationToken cancellationToken)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(cancellationToken: cancellationToken));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(IProgress<ImapMessageFetchProgressInfo> progress)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(progress: progress));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(CancellationToken cancellationToken,
                                            IProgress<ImapMessageFetchProgressInfo> progress)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(cancellationToken: cancellationToken, progress: progress));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(ImapMessageFetchBodyOptions options)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(options: options));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(ImapMessageFetchBodyOptions options,
                                            CancellationToken cancellationToken)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(options: options, cancellationToken: cancellationToken));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(ImapMessageFetchBodyOptions options,
                                            EncodingSelectionCallback selectFallbackCharset)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(options: options, selectFallbackCharset: selectFallbackCharset));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(ImapMessageFetchBodyOptions options,
                                            IProgress<ImapMessageFetchProgressInfo> progress)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(options: options, progress: progress));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(ImapMessageFetchBodyOptions options,
                                            CancellationToken cancellationToken,
                                            IProgress<ImapMessageFetchProgressInfo> progress)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(options: options, cancellationToken: cancellationToken, progress: progress));

    [Obsolete("use Body.ReadAsStringLinesAsync() instead")]
    public Task<string[]> ReadAllLinesAsync(ImapMessageFetchBodyOptions options,
                                            CancellationToken cancellationToken,
                                            IProgress<ImapMessageFetchProgressInfo> progress,
                                            EncodingSelectionCallback selectFallbackCharset)
      => ImapMessage.ToArrayAsync(Body.ReadAsStringLinesAsync(options: options, selectFallbackCharset: selectFallbackCharset, cancellationToken: cancellationToken, progress: progress));

    /*
     * ReadAllTextAsync()
     */
    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync()
      => Body.ReadAsStringAsync();

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(CancellationToken cancellationToken)
      => Body.ReadAsStringAsync(cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsStringAsync(progress: progress);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(CancellationToken cancellationToken,
                                         IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsStringAsync(cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(ImapMessageFetchBodyOptions options)
      => Body.ReadAsStringAsync(options: options);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(ImapMessageFetchBodyOptions options,
                                         CancellationToken cancellationToken)
      => Body.ReadAsStringAsync(options: options, cancellationToken: cancellationToken);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(ImapMessageFetchBodyOptions options,
                                         EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsStringAsync(options: options, selectFallbackCharset: selectFallbackCharset);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(ImapMessageFetchBodyOptions options,
                                         IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsStringAsync(options: options, progress: progress);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(ImapMessageFetchBodyOptions options,
                                         CancellationToken cancellationToken,
                                         IProgress<ImapMessageFetchProgressInfo> progress)
      => Body.ReadAsStringAsync(options: options, cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Body.ReadAsStringAsync() instead")]
    public Task<string> ReadAllTextAsync(ImapMessageFetchBodyOptions options,
                                         CancellationToken cancellationToken,
                                         IProgress<ImapMessageFetchProgressInfo> progress,
                                         EncodingSelectionCallback selectFallbackCharset)
      => Body.ReadAsStringAsync(options: options, selectFallbackCharset: selectFallbackCharset, cancellationToken: cancellationToken, progress: progress);

    [Obsolete("use Header.ReadAsString() instead")]
    public string GetRawHeader()
      => Header.ReadAsString();

    [Obsolete("use Header.ReadAsNameValuePairs() instead")]
    public IReadOnlyList<KeyValuePair<string, string>> GetHeader()
      => Header.ReadAsNameValuePairs();

    [Obsolete("use Header.ReadAsNameValuePairs() instead")]
    public IReadOnlyList<KeyValuePair<string, string>> GetHeader(bool decode)
      => Header.ReadAsNameValuePairs(decodeHeaderFieldValues: decode);

    [Obsolete("use Header.ReadAsStringAsync() instead")]
    public Task<string> GetRawHeaderAsync()
      => Header.ReadAsStringAsync();

    [Obsolete("use Header.ReadAsNameValuePairsAsync() instead")]
    public Task<IReadOnlyList<KeyValuePair<string, string>>> GetHeaderAsync()
      => Header.ReadAsNameValuePairsAsync();

    [Obsolete("use Header.ReadAsNameValuePairsAsync() instead")]
    public Task<IReadOnlyList<KeyValuePair<string, string>>> GetHeaderAsync(bool decode)
      => Header.ReadAsNameValuePairsAsync(decodeHeaderFieldValues: decode);
  }
}

