// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Smdn.Net.MessageAccessProtocols.Strings;
using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.DataStructures.MessageAttributes {
  public sealed class ImapMessageDynamicAttributes :
    ImapMessageAttributesBase,
    IImapMessageDynamicAttributes
  {
    public IReadOnlyStringSet<ImapMessageFlag> Flags { get; internal set; }

    /*
     * RFC 4551 - IMAP Extension for Conditional STORE Operation or Quick Flag Changes Resynchronization
     * http://tools.ietf.org/html/rfc4551
     */
    [CLSCompliant(false)]
    public ulong ModSeq { get; internal set; }

    internal ImapMessageDynamicAttributes(long sequence)
      : base(sequence)
    {
      this.Flags = ImapMessageFlagSet.Empty;
      this.ModSeq = 0UL; // invalid mod-sequence number;
    }

    internal override ImapMessageDynamicAttributes GetDynamicAttributes() => this;
    internal override ImapMessageStaticAttributes GetStaticAttributes() => null;
    internal override ImapMessageBodyAttributes GetBodyAttributes() => null;

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(Sequence), Sequence),
        (nameof(Flags), Flags),
        (nameof(ModSeq), ModSeq),
      });
    }
  }
}
