// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;

using Smdn.Net.MessageAccessProtocols.Strings;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  public class ImapAppendMessage : IImapAppendMessage {
    public long Length { get; }
    public DateTimeOffset? InternalDate { get; }
    public IReadOnlyStringSet<ImapMessageFlag> Flags { get; }

    public ImapAppendMessage(byte[] message,
                             DateTimeOffset? internalDate = null,
                             IEnumerable<ImapMessageFlag> flags = null)
      : this(new MemoryStream(message, false), message.Length, internalDate, flags)
    {
    }

    public ImapAppendMessage(Stream messageStream,
                             long length,
                             DateTimeOffset? internalDate = null,
                             IEnumerable<ImapMessageFlag> flags = null)
    {
      this.messageStream = messageStream ?? throw new ArgumentNullException(nameof(messageStream));
      Length = length < 0 ? throw ExceptionUtils.CreateArgumentMustBeNonZeroPositive(nameof(length), length) : length;
      InternalDate = internalDate;
      Flags = (flags == null) ? null : ImapMessageFlagSet.ToSet(flags);
    }

    private readonly Stream messageStream;
    public Stream GetMessageStream() => messageStream;
  }
}
