// 
// Copyright (c) 2019 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.Imap4.Urls;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  public static class ImapUidSet {
    public static IImapNumberSet All() => ImapNumberSet.All(asUidSet: true);
    public static IImapNumberSet Empty() => ImapNumberSet.Empty(asUidSet: true);
    public static IImapNumberSet From(long from) => ImapNumberSet.From(from, asUidSet: true);
    public static IImapNumberSet To(long to) => ImapNumberSet.To(to, asUidSet: true);
    public static IImapNumberSet Range(long from, long to) => ImapNumberSet.Range(from, to, asUidSet: true);
    public static IImapNumberSet Of(long uid) => ImapNumberSet.Of(uid, asUidSet: true);
    public static IImapNumberSet Of(long uid, params long[] uids) => ImapNumberSet.Of(asUidSet: true, uid, uids);
    public static IImapNumberSet Of(IEnumerable<long> uids) => ImapNumberSet.Of(uids, asUidSet: true);
    public static IImapNumberSet Of(ArraySegment<long> uids) => ImapNumberSet.Of(uids, asUidSet: true);
#if false && NETSTANDARD2_1
    public static IImapSequenceSet Of(Span<long> uids) => new NotImplementedException();
    public static IImapSequenceSet Of(Range uidRange) => new NotImplementedException();
#endif

    public static IImapNumberSet FromUri(Uri uri)
    {
      var uid = ImapStyleUriParser.GetUid(uri ?? throw new ArgumentNullException(nameof(uri)));

      if (uid == 0L)
        throw new ArgumentException("invalid IMAP-URI form; URI has no UID", nameof(uri));

      return Of(uid);
    }
  }
}
