// 
// Copyright (c) 2019 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;

using Smdn.Net.Imap4.Protocol.DataStructures.MessageAttributes;
using Smdn.Net.Imap4.Urls;

namespace Smdn.Net.Imap4.Protocol.Session {
  partial class ImapSession {
    private class MessageAttributeCollection<TMessageAttributes> :
      Dictionary<long, TMessageAttributes>,
      IImapMessageAttributeCollection<TMessageAttributes>
      where TMessageAttributes : ImapMessageAttributesBase
    {
      public static MessageAttributeCollection<TMessageAttributes> Create(long expectedMessageCount, ImapMailboxStatus selectedMailbox)
      {
        var capacity = expectedMessageCount <= 0L ? 16L /* as default */ : expectedMessageCount;

        return new MessageAttributeCollection<TMessageAttributes>((int)Math.Min(capacity, int.MaxValue), selectedMailbox);
      }

      public MessageAttributeCollection(int capacity, ImapMailboxStatus selectedMailbox)
        : base(capacity)
      {
        this.selectedMailbox = selectedMailbox;
      }

      public TMessageAttributes Find(long sequence)
      {
        if (TryGetValue(sequence, out var messageAttribute))
          return messageAttribute;
        else
          return null;
      }

      public void Add(TMessageAttributes @value)
      {
        Add(@value.Sequence, @value);
      }

      public TMessageAttributes[] ToArray()
      {
        var ret = new TMessageAttributes[Values.Count];

        Values.CopyTo(ret, 0);

        Clear();

        foreach (var urlAttr in ret.OfType<IImapUrl>()) {
          urlAttr.SetBaseUrl(selectedMailbox.UrlBuilder);
        }

        return ret;
      }

      private readonly ImapMailboxStatus selectedMailbox;
    }
  }
}
