// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using System.Text;

using Smdn.Text.Encodings;

namespace Smdn.Net.Imap4.Protocol.Strings {
  internal sealed class ImapTextLiteralString : ImapLiteralString, IImapStringValue {
    protected internal override long Length => EnsureStreamCreated().Length;
    protected internal override Stream GetStream() => EnsureStreamCreated();
    protected override string GetValueString() => value;
    string IImapStringValue.Value => value;
    byte[] IImapStringValue.GetBytes() => throw new NotSupportedException();

    internal Encoding Charset {
      get { return charset; }
      set {
        charset = value ?? throw new ArgumentNullException(nameof(Charset));

        // dispose previously created stream
        stream?.Dispose();
        stream = null;
      }
    }

    internal ImapTextLiteralString(string @value)
      : this(@value, OctetEncoding.SevenBits, ImapLiteralOptions.Default)
    {
    }

    internal ImapTextLiteralString(string value, Encoding charset, ImapLiteralOptions options)
      : base(options)
    {
      this.value = value ?? throw new ArgumentNullException(nameof(value));
      this.charset = charset ?? OctetEncoding.SevenBits;
    }

    private Stream EnsureStreamCreated()
    {
      if (stream == null)
        stream = new MemoryStream(charset.GetBytes(value), false);
      else
        stream.Position = 0L;

      return stream;
    }

    private readonly string value;
    private Encoding charset;
    private Stream stream;
  }
}
