// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol.Transmission.Transaction {
  /*
   * RFC 2087 - IMAP4 QUOTA extension
   * http://tools.ietf.org/html/rfc2087
   */
  internal sealed class GetQuotaTransaction : GetSetQuotaTransactionBase {
    public GetQuotaTransaction(ImapConnection connection, IImapCommandTagGenerator commandTagGenerator)
      : base(connection, commandTagGenerator)
    {
    }

    /*
     * 4.2. GETQUOTA Command
     *    Arguments:  quota root
     * 
     *    Data:       untagged responses: QUOTA
     * 
     *    Result:     OK - getquota completed
     *                NO - getquota  error:  no  such  quota  root,  permission
     *                denied
     *                BAD - command unknown or arguments invalid
     */
    protected override string GetCommand() => "GETQUOTA";

    protected override ImapString[] GetCommandArguments()
    {
#if DEBUG
      if (!RequestArguments.ContainsKey("quota root"))
        throw CreateRequestArgumentsException("arguments 'quota root' must be setted");
#endif

      return new[] { RequestArguments["quota root"] };
    }
  }
}
