// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Trace = Smdn.Net.Imap4.Client.Trace;

namespace Smdn.Net.Imap4.Protocol.Transmission.Transaction {
  internal sealed class GreetingTransaction : ImapTransactionBase {
    public GreetingTransaction(ImapConnection connection, IImapCommandTagGenerator commandTagGenerator)
      : base(connection, commandTagGenerator)
    {
    }

    protected override string GetCommand()
    {
      Trace.Log(this, "waiting for server greeting response");

      // send nothing
      return null;
    }

    protected override void OnCommandContinuationRequestReceived(ImapCommandContinuationRequest continuationRequest)
      => throw new ImapUnexpectedResponseException("unexpected command continuation request", continuationRequest);

    protected override void OnDataResponseReceived(ImapServerDataResponse data)
      => throw new ImapUnexpectedResponseException("unexpected data response", data);

    // this will never happen; expected command tag is null
    protected override void OnTaggedStatusResponseReceived(ImapTaggedStatusResponse tagged)
      => throw new ImapUnexpectedResponseException("unexpected tagged status response", tagged);

    protected override void OnUntaggedStatusResponseReceived(ImapUntaggedStatusResponse untagged)
    {
      switch (untagged.Condition) {
        case ImapResponseCondition.Ok:
        case ImapResponseCondition.PreAuth:
        case ImapResponseCondition.Bye:
          Finish(untagged);
          break;

        default:
          // BAD or NO; inpossible?
          throw new ImapUnexpectedResponseException($"unexpected condition ({untagged.Condition})", untagged);
      }
    }
  }
}
