// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;

using Smdn.Text;
using Smdn.Text.Encodings;

namespace Smdn.Net.Imap4.Protocol.Transmission {
#if SERIALIZATION
  [Serializable]
#endif
  public sealed class ImapResponseText {
    public ImapResponseCode Code { get; }
    public IReadOnlyList<ImapData> Arguments { get; }
    public string Text { get; private set; }

    internal ImapResponseText(string text)
    {
      this.Code = null;
      this.Arguments =
#if NET45
        ArrayExtensions.Empty<ImapData>();
#else
        Array.Empty<ImapData>();
#endif
      this.Text = text ?? throw new ArgumentNullException(nameof(text));
    }

    internal ImapResponseText(ImapResponseCode code, IReadOnlyList<ImapData> arguments, string text)
    {
      this.Code = code ?? throw new ArgumentNullException(nameof(code));
      this.Arguments = arguments ?? throw new ArgumentNullException(nameof(arguments));
      this.Text = text ?? throw new ArgumentNullException(nameof(text));
    }

    /*
     * RFC 5255 - Internet Message Access Protocol Internationalization
     * http://tools.ietf.org/html/rfc5255
     */
    internal void ConvertTextToUTF8()
    {
      var bytes = OctetEncoding.EightBits.GetBytes(Text);

      Text = Encoding.UTF8.GetString(bytes);
    }

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(Code), Code),
        (nameof(Arguments), Arguments),
        (nameof(Text), Text),
      });
    }
  }
}
