// 
// Copyright (c) 2008 smdn <smdn@smdn.jp>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Text;

namespace Smdn.Net.Imap4.Protocol.Transmission {
#if SERIALIZATION
  [Serializable]
#endif
  public class ImapServerDataResponse : ImapResponse {
    internal static ImapServerDataResponse Create(IReadOnlyList<ImapData> data)
    {
      // 'Mailbox Size' or 'Message Status'
      // mailbox-data
      //   number SP "EXISTS"
      //   number SP "RECENT"
      // message-data
      //   nz-number SP "EXPUNGE"
      //   nz-number SP "FETCH" SP msg-att
      if (2 <= data.Count && data[1].Format == ImapDataFormat.Text &&
          ImapServerDataResponseType.SizeStatusTypes.TryGetValue(data[1].GetTextAsString(), out var sizeStatusType))
        return new ImapServerSizeStatusDataResponse(sizeStatusType, data[0], data.Slice(2));

      // 'Server and Mailbox Status'
      // mailbox-data
      //   "FLAGS" SP flag-list
      //   "LIST" SP mailbox-list
      //   "STATUS" SP mailbox SP "(" [status-att-list] ")"
      //   etc.
      if (1 <= data.Count && data[0].Format == ImapDataFormat.Text &&
          ImapServerDataResponseType.AllTypesExceptSizeStatus.TryGetValue(data[0].GetTextAsString(), out var type))
        return new ImapServerDataResponse(type, data.Slice(1));

      Smdn.Net.Imap4.Client.Trace.Verbose(string.Concat("unknown data response type: ", data[0]));

      return new ImapServerDataResponse(ImapServerDataResponseType.InvalidOrUnknown, data);
    }

#if SERIALIZATION
    [Serializable]
#endif
    private class ImapServerSizeStatusDataResponse : ImapServerDataResponse {
      private readonly ImapData number;
      public override ImapData SizeStatusNumber => number;

      internal ImapServerSizeStatusDataResponse(ImapServerDataResponseType type, ImapData number, IReadOnlyList<ImapData> dataList)
        : base(type, dataList)
      {
        this.number = number;
      }
    }

    /*
     * instance members
     */
    public ImapServerDataResponseType Type { get; }
    public virtual ImapData SizeStatusNumber => null;
    public IReadOnlyList<ImapData> DataList { get; }

    private ImapServerDataResponse(ImapServerDataResponseType type, IReadOnlyList<ImapData> dataList)
    {
      Type = type;
      DataList = dataList;
    }

    public override string ToString()
    {
      return StringConversion.ToString(GetType(), new (string, object)[] {
        (nameof(Type), Type),
        (nameof(SizeStatusNumber), SizeStatusNumber),
        (nameof(DataList), DataList),
      });
    }
  }
}
