using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Transmission;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  [TestFixture]
  public class ImapNamespaceTests {
    private static ImapNamespace FromResponse(string namespaceResponse)
    {
      using (var receiver = new ImapPseudoResponseReceiver(namespaceResponse)) {
        var response = receiver.ReceiveResponse() as ImapServerDataResponse;

        return ImapServerDataResponseParser.FromNamespace(response);
      }
    }

    [Test]
    public void TestNamespaceDescListProperties_NonNull()
    {
      var ns = FromResponse("* NAMESPACE NIL NIL NIL\r\n");

      Assert.IsNotNull(ns.PersonalNamespaces, nameof(ns.PersonalNamespaces));
      Assert.IsNotNull(ns.OtherUsersNamespaces, nameof(ns.OtherUsersNamespaces));
      Assert.IsNotNull(ns.SharedNamespaces, nameof(ns.SharedNamespaces));

      CollectionAssert.IsEmpty(ns.PersonalNamespaces, nameof(ns.PersonalNamespaces));
      CollectionAssert.IsEmpty(ns.OtherUsersNamespaces, nameof(ns.OtherUsersNamespaces));
      CollectionAssert.IsEmpty(ns.SharedNamespaces, nameof(ns.SharedNamespaces));
    }

    [TestCase("* NAMESPACE NIL NIL NIL\r\n")]
    [TestCase("* NAMESPACE ((\"\" \"/\")) ((\"~\" \"/\")) NIL\r\n")]
    public void TestToString(string namespaceResponse)
    {
      var ns = FromResponse(namespaceResponse);

      Assert.DoesNotThrow(() => ns.ToString());
    }
  }
}
