using System;
using System.Collections.Generic;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Transmission;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  [TestFixture]
  public class ImapNamespaceDescTests {
    private static ImapNamespaceDesc FromResponse(string namespaceResponse)
    {
      using (var receiver = new ImapPseudoResponseReceiver(namespaceResponse)) {
        var response = receiver.ReceiveResponse() as ImapServerDataResponse;

        var ns = ImapServerDataResponseParser.FromNamespace(response);

        return ns.PersonalNamespaces[0];
      }
    }

    [Test]
    public void TestExtensions()
    {
      var desc = FromResponse($"* NAMESPACE ((\"#mh/\" \"/\" \"X-PARAM\" (\"FLAG1\" \"FLAG2\"))) NIL NIL\r\n");

      Assert.AreEqual("#mh/", desc.Prefix);
      Assert.AreEqual("#mh/", desc.TranslatedPrefix);
      Assert.AreEqual("/", desc.HierarchyDelimiter);
      Assert.IsNotNull(desc.Extensions);
      Assert.AreEqual(1, desc.Extensions.Count);
      Assert.IsTrue(desc.Extensions.ContainsKey("X-PARAM"));
      Assert.IsTrue(desc.Extensions.ContainsKey("x-param"));
      Assert.IsNotNull(desc.Extensions["X-PARAM"]);
      Assert.AreEqual(2, desc.Extensions["X-PARAM"].Count);
      Assert.AreEqual("FLAG1", desc.Extensions["X-PARAM"][0]);
      Assert.AreEqual("FLAG2", desc.Extensions["X-PARAM"][1]);
    }

    [Test]
    public void TestNoExtensions()
    {
      var desc = FromResponse("* NAMESPACE ((\"Other Users/\" \"/\")) NIL NIL\r\n");

      Assert.AreEqual("Other Users/", desc.Prefix);
      Assert.AreEqual("Other Users/", desc.TranslatedPrefix);
      Assert.AreEqual("/", desc.HierarchyDelimiter);
      Assert.IsNotNull(desc.Extensions);
      Assert.AreEqual(0, desc.Extensions.Count);
    }

    [TestCase("Andere Ben&APw-tzer", "Andere Benützer")]
    [TestCase("Gemeinsame Postf&AM8-cher", "Gemeinsame PostfÏcher")]
    public void TestTranslatedPrefix(string respTranslatedPrefix, string expectedTranslatedPrefix)
    {
      var desc = FromResponse($"* NAMESPACE ((\"Other Users/\" \"/\" \"TRANSLATION\" (\"{respTranslatedPrefix}\"))) NIL NIL\r\n");

      Assert.AreEqual("Other Users/", desc.Prefix);
      Assert.AreEqual(expectedTranslatedPrefix, desc.TranslatedPrefix);
    }

    [Test]
    public void TestToString()
    {
      var desc = FromResponse($"* NAMESPACE ((\"#mh/\" \"/\" \"X-PARAM\" (\"FLAG1\" \"FLAG2\"))) NIL NIL\r\n");

      Assert.DoesNotThrow(() => desc.ToString());
    }
  }
}
