using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol.DataStructures {
  [TestFixture]
  public class ImapQuotaResourceLimitTests {
    [TestCase("resource", 0L)]
    [TestCase("resource", 1L)]
    [TestCase("resource", 0xffffffff)]
    public void TestConstruct(string name, long limit)
    {
      Assert.DoesNotThrow(() => new ImapQuotaResourceLimit(name, limit));
    }

    [Test]
    public void TestConstruct_Name_ArgumentException()
    {
      var ex1 = Assert.Throws<ArgumentNullException>(() => new ImapQuotaResourceLimit(null, 1L));

      Assert.AreEqual("name", ex1.ParamName);

      var ex2 = Assert.Throws<ArgumentException>(() => new ImapQuotaResourceLimit(string.Empty, 1L));

      Assert.AreEqual("name", ex2.ParamName);
    }

    [Test]
    public void TestConstruct_Limit_ArgumentOutOfRangeException()
    {
      foreach (var create in new Action[] {
        () => new ImapQuotaResourceLimit("name", -1L),
        () => new ImapQuotaResourceLimit("name", 0x100000000),
        () => new ImapQuotaResourceLimit("name", long.MaxValue),
      }) {
        var ex = Assert.Throws<ArgumentOutOfRangeException>(() => create());

        Assert.AreEqual("limit", ex.ParamName);
      }
    }

    [Test]
    public void TestToString()
    {
      var quotaResource = new ImapQuotaResourceLimit("storage", 24L);

      Assert.DoesNotThrow(() => quotaResource.ToString());
    }
  }
}

