using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol.Transmission {
  [TestFixture]
  public class ImapCommandContinuationRequestTests {
    private ImapCommandContinuationRequest CreateFromResponse(string response)
    {
      using (var receiver = new ImapPseudoResponseReceiver(response)) {
        return receiver.ReceiveResponse() as ImapCommandContinuationRequest;
      }
    }

    [Test]
    public void TestSerializeBinary_TextOrBase64()
    {
      var resp = CreateFromResponse("+ Ready for additional command text\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(resp, deserialized => {
        Assert.AreEqual("Ready for additional command text", deserialized.Text);
        Assert.IsNull(deserialized.ResponseText);
      });
    }

    [Test]
    public void TestSerializeBinary_RespText()
    {
      var resp = CreateFromResponse("+ [SERVERBUG] This should not happen\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(resp, deserialized => {
        Assert.IsNull(deserialized.Text);
        Assert.IsNotNull(deserialized.ResponseText);
        Assert.AreEqual(ImapResponseCode.ServerBug, deserialized.ResponseText.Code);
        Assert.IsEmpty(deserialized.ResponseText.Arguments);
        Assert.AreEqual("This should not happen", deserialized.ResponseText.Text);
      });
    }
  }
}
