using NUnit.Framework;

using System;
using System.IO;
using System.Text;

using Smdn.IO.Streams.LineOriented;
using Smdn.Net.Imap4.Protocol.Strings;
using Smdn.Text.Encodings;

namespace Smdn.Net.Imap4.Protocol.Transmission {
  [TestFixture]
  public class ImapCommandSenderTests {
    [SetUp]
    public void Setup()
    {
      baseStream = new MemoryStream();
      sender = new ImapCommandSender(new LineOrientedBufferedStream(baseStream));
    }

    [TearDown]
    public void TearDown()
    {
      // nothing to do
    }

    private string SendCommand(ImapCommand command)
    {
      baseStream.Seek(0, SeekOrigin.Begin);

      sender.Send(command);

      while (sender.IsCommandContinuing) {
        sender.SendRemainder();
      }

      baseStream.Seek(0, SeekOrigin.Begin);

      var reader = new StreamReader(baseStream, OctetEncoding.EightBits);

      return reader.ReadToEnd();
    }

    [Test]
    public void TestSend_Command()
    {
      var command = ImapCommand.Create("login", "0001", "username", "password");

      Assert.AreEqual("0001 login username password\r\n",
                      SendCommand(command));
    }

    [Test]
    public void TestSend_Command_NoArguments()
    {
      var command = ImapCommand.Create("NOOP", "0001");

      Assert.AreEqual("0001 NOOP\r\n",
                      SendCommand(command));
    }

    [Test]
    public void TestSend_ContinuationRequest()
    {
      var command = ImapCommand.CreateContinuation("xxxxxxxx");

      Assert.AreEqual("xxxxxxxx\r\n",
                      SendCommand(command));
    }

    [Test]
    public void TestSend_Command_ContainsSynchronizingLiteral()
    {
      var command = ImapCommand.Create("SEARCH", "0001", "FROM", ImapLiteralString.Create("from", Encoding.ASCII, ImapLiteralOptions.Synchronizing));

      Assert.AreEqual("0001 SEARCH FROM {4}\r\nfrom\r\n",
                      SendCommand(command));
    }

    [Test]
    public void TestSend_Command_ContainsNonSynchronizingLiteral()
    {
      var command = ImapCommand.Create("SEARCH", "0001", "FROM", ImapLiteralString.Create("from", Encoding.ASCII, ImapLiteralOptions.NonSynchronizing));

      Assert.AreEqual("0001 SEARCH FROM {4+}\r\nfrom\r\n",
                      SendCommand(command));
    }

    private MemoryStream baseStream;
    private ImapCommandSender sender;
  }
}
