using System;
using System.Collections.Generic;
using NUnit.Framework;

using Smdn.Collections;

namespace Smdn.Net.Imap4.Protocol.Transmission {
  [TestFixture]
  public class ImapMalformedDataExceptionTests {
    [Test]
    public void TestSerializeBinary_CausedDataNull()
    {
      var ex = new ImapMalformedDataException();

      Assert.IsNull(ex.CausedData);

      TestUtils.Assert.IsSerializableBinaryFormat(ex, delegate(ImapMalformedDataException deserialized) {
        Assert.IsNull(deserialized.CausedData);
      });
    }

    private IReadOnlyList<ImapData> CreateData(string text)
    {
      using (var receiver = new ImapPseudoResponseReceiver("* X-DUMMY " + text + "\r\n")) {
        return (receiver.ReceiveResponse() as ImapServerDataResponse).DataList.Slice(1);
      }
    }

    [Test]
    public void TestSerializeBinary_CausedDataNotNull()
    {
      var ex = new ImapMalformedDataException(CreateData("exception")[0]);

      Assert.IsNotNull(ex.CausedData);

      TestUtils.Assert.IsSerializableBinaryFormat(ex, delegate(ImapMalformedDataException deserialized) {
        Assert.IsNotNull(deserialized.CausedData);
        Assert.AreEqual("exception", deserialized.CausedData.GetTextAsString());
      });
    }
  }
}
