using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol.Transmission {
  [TestFixture]
  public class ImapResponseTextTests {
    private ImapResponseText CreateFromResponse(string response)
    {
      using (var receiver = new ImapPseudoResponseReceiver(response)) {
        return (receiver.ReceiveResponse() as ImapStatusResponse).ResponseText;
      }
    }

    [Test]
    public void TestSerializeBinary()
    {
      var respText = CreateFromResponse("A142 OK [READ-WRITE] SELECT completed\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(respText, delegate (ImapResponseText deserialized) {
        Assert.AreEqual(ImapResponseCode.ReadWrite, deserialized.Code);
        Assert.AreEqual("SELECT completed", deserialized.Text);
        Assert.IsNotNull(deserialized.Arguments);
        Assert.AreEqual(0, deserialized.Arguments.Count);
      });
    }

    [Test]
    public void TestSerializeBinary_ResponseCodeWithArguments()
    {
      var respText = CreateFromResponse("* OK [UIDVALIDITY 988028003] UID validity status\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(respText, delegate(ImapResponseText deserialized) {
        Assert.AreEqual(ImapResponseCode.UidValidity, deserialized.Code);
        Assert.AreEqual("UID validity status", deserialized.Text);
        Assert.IsNotNull(deserialized.Arguments);
        Assert.AreEqual(1, deserialized.Arguments.Count);
        Assert.AreEqual("988028003", deserialized.Arguments[0].GetTextAsString());
      });
    }

    [Test]
    public void TestSerializeBinary_ResponseCodeWithSubCode()
    {
      var respText = CreateFromResponse("a NO [METADATA TOOMANY] SETMETADATA failed\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(respText, delegate (ImapResponseText deserialized) {
        Assert.AreEqual(ImapResponseCode.MetadataTooMany, deserialized.Code);
        Assert.AreEqual("SETMETADATA failed", deserialized.Text);
        Assert.IsNotNull(deserialized.Arguments);
        Assert.AreEqual(0, deserialized.Arguments.Count);
      });
    }

    [Test]
    public void TestSerializeBinary_ResponseCodeWithSubCodeAndArguments()
    {
      var respText = CreateFromResponse("a OK [METADATA LONGENTRIES 2199] GETMETADATA complete\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(respText, delegate(ImapResponseText deserialized) {
        Assert.AreEqual(ImapResponseCode.MetadataLongEntries, deserialized.Code);
        Assert.AreEqual("GETMETADATA complete", deserialized.Text);
        Assert.IsNotNull(deserialized.Arguments);
        Assert.AreEqual(1, deserialized.Arguments.Count);
        Assert.AreEqual("2199", deserialized.Arguments[0].GetTextAsString());
      });
    }

    [Test]
    public void TestSerializeBinary_Unknown()
    {
      var respText = CreateFromResponse("a OK [X-UNKNOWN ARG1 ARG2] X-EXTENSION complete\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(respText, delegate (ImapResponseText deserialized) {
        Assert.IsNotNull(deserialized.Code);
        Assert.AreEqual("X-UNKNOWN", deserialized.Code);
        Assert.AreEqual("X-EXTENSION complete", deserialized.Text);
        Assert.IsNotNull(deserialized.Arguments);
        Assert.AreEqual(2, deserialized.Arguments.Count);
        Assert.AreEqual("ARG1", deserialized.Arguments[0].GetTextAsString());
        Assert.AreEqual("ARG2", deserialized.Arguments[1].GetTextAsString());
      });
    }
  }
}
