using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol.Transmission {
  [TestFixture]
  public class ImapServerDataResponseTests {
    private ImapServerDataResponse CreateFromResponse(string response)
    {
      using (var receiver = new ImapPseudoResponseReceiver(response)) {
        return receiver.ReceiveResponse() as ImapServerDataResponse;
      }
    }

    [Test]
    public void TestSerializeBinary()
    {
      var resp = CreateFromResponse("* LIST () \"\" INBOX\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(resp, deserialized => {
        Assert.AreEqual(ImapServerDataResponseType.List, deserialized.Type);

        Assert.IsNull(deserialized.SizeStatusNumber);

        Assert.IsNotNull(deserialized.DataList);
        Assert.AreEqual(3, deserialized.DataList.Count);
        Assert.AreEqual(ImapDataFormat.List, deserialized.DataList[0].Format);
        Assert.AreEqual(0, deserialized.DataList[0].List.Count);
        Assert.AreEqual(ImapDataFormat.Text, deserialized.DataList[1].Format);
        Assert.AreEqual(string.Empty, deserialized.DataList[1].GetTextAsString());
        Assert.AreEqual(ImapDataFormat.Text, deserialized.DataList[2].Format);
        Assert.AreEqual("INBOX", deserialized.DataList[2].GetTextAsString());
      });
    }

    [Test]
    public void TestSerializeBinary_SizeStatus()
    {
      var resp = CreateFromResponse("* 3 EXISTS\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(resp, deserialized => {
        Assert.AreEqual(ImapServerDataResponseType.Exists, deserialized.Type);

        Assert.IsNotNull(deserialized.SizeStatusNumber);
        Assert.AreEqual(ImapDataFormat.Text, deserialized.SizeStatusNumber.Format);
        Assert.AreEqual("3", deserialized.SizeStatusNumber.GetTextAsString());

        Assert.IsNotNull(deserialized.DataList);
        Assert.AreEqual(0, deserialized.DataList.Count);
      });
    }

    [Test]
    public void TestSerializeBinary_SizeStatus_WithDataList()
    {
      var resp = CreateFromResponse("* 1 FETCH (UID 2)\r\n");

      TestUtils.Assert.IsSerializableBinaryFormat(resp, deserialized => {
        Assert.AreEqual(ImapServerDataResponseType.Fetch, deserialized.Type);

        Assert.IsNotNull(deserialized.SizeStatusNumber);
        Assert.AreEqual(ImapDataFormat.Text, deserialized.SizeStatusNumber.Format);
        Assert.AreEqual("1", deserialized.SizeStatusNumber.GetTextAsString());

        Assert.IsNotNull(deserialized.DataList);
        Assert.AreEqual(1, deserialized.DataList.Count);
        Assert.AreEqual(ImapDataFormat.List, deserialized.DataList[0].Format);
        Assert.AreEqual(2, deserialized.DataList[0].List.Count);
        Assert.AreEqual(ImapDataFormat.Text, deserialized.DataList[0].List[0].Format);
        Assert.AreEqual("UID", deserialized.DataList[0].List[0].GetTextAsString());
        Assert.AreEqual(ImapDataFormat.Text, deserialized.DataList[0].List[1].Format);
        Assert.AreEqual("2", deserialized.DataList[0].List[1].GetTextAsString());
      });
    }
  }
}
