using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapCollationAlgorithmTests {
    [Test]
    public void TestEquals()
    {
      var x = ImapCollationAlgorithm.Default;

      Assert.IsTrue(x.Equals(ImapCollationAlgorithm.Default));
      Assert.IsTrue(x.Equals(new ImapCollationAlgorithm("DEFAULT")));
      Assert.IsTrue(x.Equals((object)"DEFAULT"));
      Assert.IsTrue(x.Equals((object)"default"));
      Assert.IsTrue(x.Equals((object)ImapCollationAlgorithm.Default));
      Assert.IsTrue(x.Equals("DEFAULT"));
      Assert.IsTrue(x.Equals("default"));
      Assert.IsTrue(x.Equals(ImapCollationAlgorithm.Default.ToString()));

      Assert.IsFalse(x.Equals(ImapCollationAlgorithm.Octet));
      Assert.IsFalse(x.Equals((ImapString)ImapCollationAlgorithm.Default.Value));
      Assert.IsFalse(x.Equals(new ImapCapability("DEFAULT")));
      Assert.IsFalse(x.Equals((object)new ImapCapability("DEFAULT")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = ImapCollationAlgorithm.Default;

      Assert.IsTrue(x == ImapCollationAlgorithm.Default);
      Assert.IsTrue(x == new ImapCollationAlgorithm("DEFAULT"));
      Assert.IsFalse(x != ImapCollationAlgorithm.Default);
      Assert.IsFalse(x != new ImapCollationAlgorithm("DEFAULT"));

      Assert.IsFalse(x == ImapCollationAlgorithm.Octet);
      Assert.IsFalse(x == new ImapCollationAlgorithm("i;octet"));
      Assert.IsTrue(x != ImapCollationAlgorithm.Octet);
      Assert.IsTrue(x != new ImapCollationAlgorithm("i;octet"));
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("default", ImapCollationAlgorithm.Default.ToString());
      Assert.AreEqual("i;octet", ImapCollationAlgorithm.Octet.ToString());
    }

    [Test]
    public void TestSerializeBinary()
    {
      TestUtils.Assert.IsSerializableBinaryFormat(ImapCollationAlgorithm.Default, deserialized => {
        Assert.AreNotSame(ImapCollationAlgorithm.Default, deserialized);
        Assert.AreEqual(ImapCollationAlgorithm.Default, deserialized);
      });
    }
  }
}
