using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Transmission;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapDateTimeFormatTests {
    [Test]
    public void TestToDateString()
    {
      Assert.AreEqual("1-Feb-1994", ImapDateTimeFormat.ToDateString(new DateTime(1994, 2, 1)));
      Assert.AreEqual("11-Feb-1994", ImapDateTimeFormat.ToDateString(new DateTime(1994, 2, 11)));
      Assert.AreEqual("1-Feb-2000", ImapDateTimeFormat.ToDateString(new DateTime(2000, 2, 1)));
    }

    [Test]
    public void TestToDateTimeString()
    {
      Assert.AreEqual("\"17-Jul-1996 02:44:25 -0700\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(-7))));
      Assert.AreEqual("\"07-Jul-1996 02:44:25 -0700\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7,  7, 2, 44, 25, TimeSpan.FromHours(-7))));
      Assert.AreEqual("\"17-Jul-1996 02:44:25 +0700\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(+7))));
      Assert.AreEqual("\"17-Jul-1996 02:44:25 +0000\"", ImapDateTimeFormat.ToDateTimeString(new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.Zero)));
    }

    [Test]
    public void TestTryParse()
    {
      foreach (var testcase in new[] {
        (input: "17-Jul-1996 02:44:25 +0000", expected: new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.Zero)),
        (input: "17-Jul-1996 02:44:25 -0000", expected: new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.Zero)),
        (input: "17-Jul-1996 02:44:25 -0700", expected: new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(-7))),
        (input: "17-Jul-1996 02:44:25 -0730", expected: new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(-7.5))),
        (input: "17-Jul-1996 02:44:25 +0700", expected: new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(+7))),
        (input: "17-Jul-1996 02:44:25 +0730", expected: new DateTimeOffset(1996, 7, 17, 2, 44, 25, TimeSpan.FromHours(+7.5))),
      }) {
        Assert.IsTrue(ImapDateTimeFormat.TryParse(testcase.input, out var actual), testcase.input);
        Assert.AreEqual(testcase.expected, actual, testcase.input);
      }
    }

    [Test]
    public void TestTryParse_StartsWithSpace()
    {
      foreach (var testcase in new[] {
        (input: " 9-Mar-2010 00:58:29 -0500", expected: new DateTimeOffset(2010, 3, 9, 0, 58, 29, TimeSpan.FromHours(-5))),
      }) {
        Assert.IsTrue(ImapDateTimeFormat.TryParse(testcase.input, out var actual), testcase.input);
        Assert.AreEqual(testcase.expected, actual, testcase.input);
      }
    }

    [Test]
    public void TestTryParse_InvalidFormat()
    {
      foreach (var input in new[] {
        "invalid date and time",
        " 9-Mar-10 00:58:29 -0500",
        "99-Mar-2010 00:58:29 -0500",
        "19-Mar-12010 00:58:29 -0500",
        " 9-Mar-2010 00:58:29 0500",
      }) {
        Assert.IsFalse(ImapDateTimeFormat.TryParse(input, out _), input);
      }
    }
  }
}
