using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapMailboxFlagTests {
    [Test]
    public void TestEquals()
    {
      var x = ImapMailboxFlag.Flagged;

      Assert.IsTrue(x.Equals(ImapMailboxFlag.Flagged));
      Assert.IsTrue(x.Equals((object)"\\Flagged"));
      Assert.IsTrue(x.Equals((object)"\\FLAGGED"));
      Assert.IsTrue(x.Equals((object)ImapMailboxFlag.Flagged));
      Assert.IsTrue(x.Equals("\\Flagged"));
      Assert.IsTrue(x.Equals("\\FLAGGED"));
      Assert.IsTrue(x.Equals(ImapMailboxFlag.Flagged.ToString()));

      Assert.IsFalse(x.Equals(ImapMailboxFlag.Marked));
      Assert.IsFalse(x.Equals((ImapString)ImapMailboxFlag.Flagged.Value));
      Assert.IsFalse(x.Equals(new ImapCapability("\\Flagged")));
      Assert.IsFalse(x.Equals((object)new ImapCapability("\\Flagged")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = ImapMailboxFlag.Flagged;

      Assert.IsTrue(x == ImapMailboxFlag.Flagged);
      Assert.IsFalse(x != ImapMailboxFlag.Flagged);

      Assert.IsFalse(x == ImapMailboxFlag.Marked);
      Assert.IsTrue(x != ImapMailboxFlag.Marked);
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("\\Flagged", ImapMailboxFlag.Flagged.ToString());
      Assert.AreEqual("\\Marked", ImapMailboxFlag.Marked.ToString());
    }

    [Test]
    public void TestSerializeBinary()
    {
      TestUtils.Assert.IsSerializableBinaryFormat(ImapMailboxFlag.Flagged, deserialized => {
        Assert.AreNotSame(ImapMailboxFlag.Flagged, deserialized);
        Assert.AreEqual(ImapMailboxFlag.Flagged, deserialized);
      });
    }
  }
}
