using System;
using System.Collections.Generic;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapMailboxNamesTests {
    [Test]
    public void TestEqualityComparer_GetHashCode_Inbox()
    {
      Assert.AreEqual(ImapMailboxNames.EqualityComparer.GetHashCode("INBOX"),
                      ImapMailboxNames.EqualityComparer.GetHashCode("inbox"));

      Assert.AreEqual(ImapMailboxNames.EqualityComparer.GetHashCode("INBOX"),
                      ImapMailboxNames.EqualityComparer.GetHashCode("Inbox"));

      Assert.AreNotEqual(ImapMailboxNames.EqualityComparer.GetHashCode("INBOX"),
                         ImapMailboxNames.EqualityComparer.GetHashCode("hoge"));
    }

    [Test]
    public void TestEqualityComparer_GetHashCode()
    {
      Assert.AreNotEqual(ImapMailboxNames.EqualityComparer.GetHashCode("HOGE"),
                         ImapMailboxNames.EqualityComparer.GetHashCode("hoge"));

      Assert.AreNotEqual(ImapMailboxNames.EqualityComparer.GetHashCode("HOGE"),
                         ImapMailboxNames.EqualityComparer.GetHashCode("Hoge"));

      Assert.AreNotEqual(ImapMailboxNames.EqualityComparer.GetHashCode("Hoge"),
                         ImapMailboxNames.EqualityComparer.GetHashCode("hoge"));

      Assert.DoesNotThrow(() => ImapMailboxNames.EqualityComparer.GetHashCode(string.Empty));

      Assert.Throws<ArgumentNullException>(() => ImapMailboxNames.EqualityComparer.GetHashCode(null));
    }

    [Test]
    public void TestEqualityComparer_Equals_Inbox()
    {
      Assert.IsTrue(ImapMailboxNames.EqualityComparer.Equals("INBOX", "INBOX"));
      Assert.IsTrue(ImapMailboxNames.EqualityComparer.Equals("INBOX", "inbox"));
      Assert.IsTrue(ImapMailboxNames.EqualityComparer.Equals("INBOX", "Inbox"));
      Assert.IsTrue(ImapMailboxNames.EqualityComparer.Equals("inbox", "INBOX"));
      Assert.IsTrue(ImapMailboxNames.EqualityComparer.Equals("Inbox", "INBOX"));

      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("INBOX", "hoge"));
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("hoge", "INBOX"));
    }

    [Test]
    public void TestEqualityComparer_Equals()
    {
      Assert.IsTrue (ImapMailboxNames.EqualityComparer.Equals(null, null));
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals(null, "hoge"));
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("hoge", null));

      Assert.IsTrue (ImapMailboxNames.EqualityComparer.Equals(string.Empty, string.Empty));
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals(string.Empty, "hoge"));
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("hoge", string.Empty));

      Assert.IsTrue (ImapMailboxNames.EqualityComparer.Equals("hoge", "hoge"));
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("Hoge", "hoge"));

      Assert.IsTrue (ImapMailboxNames.EqualityComparer.Equals("INBOX\u3007", "INBOX\u3007"), "StringComparison");
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("INBOX\u3007", "INBOX"), "StringComparison");
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("INBOX", "INBOX\u3007"), "StringComparison");
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("INBOX\u3007", "\u3007INBOX"), "StringComparison");

      Assert.IsTrue (ImapMailboxNames.EqualityComparer.Equals("hoge\u3007", "hoge\u3007"), "StringComparison");
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("HOGE\u3007", "hoge\u3007"), "StringComparison");
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("hoge\u3007", "hoge"), "StringComparison");
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("hoge", "hoge\u3007"), "StringComparison");
      Assert.IsFalse(ImapMailboxNames.EqualityComparer.Equals("hoge\u3007", "\u3007hoge"), "StringComparison");
    }

    [Test]
    public void TestEqualityComparer()
    {
      var s = new HashSet<string>(ImapMailboxNames.EqualityComparer);

      s.Add("inbox"); Assert.AreEqual(1, s.Count);
      s.Add("Inbox"); Assert.AreEqual(1, s.Count);
      s.Add("INBOX"); Assert.AreEqual(1, s.Count);

      s.Add("hoge"); Assert.AreEqual(2, s.Count);
      s.Add("HOGE"); Assert.AreEqual(3, s.Count);

      s.Add(string.Empty); Assert.AreEqual(4, s.Count);
      s.Add(null); Assert.AreEqual(5, s.Count);
    }

    [Test]
    public void TestIsInbox()
    {
      Assert.IsTrue(ImapMailboxNames.IsInbox("INBOX"));
      Assert.IsTrue(ImapMailboxNames.IsInbox("inbox"));
      Assert.IsTrue(ImapMailboxNames.IsInbox("Inbox"));

      Assert.IsFalse(ImapMailboxNames.IsInbox(null));
      Assert.IsFalse(ImapMailboxNames.IsInbox(string.Empty));

      Assert.IsFalse(ImapMailboxNames.IsInbox("INBOX2"));
      Assert.IsFalse(ImapMailboxNames.IsInbox("inbox2"));
      Assert.IsFalse(ImapMailboxNames.IsInbox("mailbox"));
    }
  }
}
