using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapMessageFlagTests {
    [Test]
    public void TestEquals()
    {
      var x = ImapMessageFlag.Recent;

      Assert.IsTrue(x.Equals(ImapMessageFlag.Recent));
      Assert.IsTrue(x.Equals((object)"\\Recent"));
      Assert.IsTrue(x.Equals((object)"\\RECENT"));
      Assert.IsTrue(x.Equals((object)ImapMessageFlag.Recent));
      Assert.IsTrue(x.Equals("\\Recent"));
      Assert.IsTrue(x.Equals("\\RECENT"));
      Assert.IsTrue(x.Equals(ImapMessageFlag.Recent.ToString()));

      Assert.IsFalse(x.Equals(ImapMessageFlag.Deleted));
      Assert.IsFalse(x.Equals((ImapString)ImapMessageFlag.Recent.Value));
      Assert.IsFalse(x.Equals(new ImapCapability("\\Recent")));
      Assert.IsFalse(x.Equals((object)new ImapCapability("\\Recent")));

      var y = new ImapMessageFlag("$label1");
      var o = y;

      Assert.IsTrue(y.Equals(o));
      Assert.IsTrue(y.Equals(new ImapMessageFlag("$label1")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = ImapMessageFlag.Recent;

      Assert.IsTrue(x == ImapMessageFlag.Recent);
      Assert.IsFalse(x != ImapMessageFlag.Recent);

      Assert.IsFalse(x == ImapMessageFlag.Deleted);
      Assert.IsTrue(x != ImapMessageFlag.Deleted);

      var y = new ImapMessageFlag("$label1");

      Assert.IsTrue(y == new ImapMessageFlag("$label1"));
      Assert.IsFalse(y != new ImapMessageFlag("$label1"));

      Assert.IsFalse(y == new ImapMessageFlag("$label2"));
      Assert.IsTrue(y != new ImapMessageFlag("$label2"));
    }

    [Test]
    public void TestGetValidatedKeyword_Null()
    {
      Assert.Throws<ArgumentNullException>(() => ImapMessageFlag.GetValidatedKeyword(null));
    }

    [Test]
    public void TestGetValidatedKeyword_Empty()
    {
      Assert.Throws<ArgumentException>(() => ImapMessageFlag.GetValidatedKeyword(string.Empty));
    }

    [Test]
    public void TestGetValidatedKeyword_ContainsCtrl()
    {
      Assert.Throws<ArgumentException>(() => ImapMessageFlag.GetValidatedKeyword("keyword\n"));
    }

    [Test]
    public void TestGetValidatedKeyword_SystemFlags()
    {
      Assert.Throws<ArgumentException>(() => ImapMessageFlag.GetValidatedKeyword("\\Deleted"));
    }

    [Test]
    public void TestIsValidKeyword()
    {
      Assert.IsTrue(ImapMessageFlag.IsValidKeyword("a"));
      Assert.IsTrue(ImapMessageFlag.IsValidKeyword("$label"));
      Assert.IsTrue(ImapMessageFlag.IsValidKeyword("<label>"));
      Assert.IsFalse(ImapMessageFlag.IsValidKeyword(string.Empty));
      Assert.IsFalse(ImapMessageFlag.IsValidKeyword("keyword\n"));
      Assert.IsFalse(ImapMessageFlag.IsValidKeyword("\\Deleted"));
    }

    [Test]
    public void TestGetValidatedKeyword_Valid()
    {
      Assert.AreEqual("a", ImapMessageFlag.GetValidatedKeyword("a"));
      Assert.AreEqual("$label", ImapMessageFlag.GetValidatedKeyword("$label"));
      Assert.AreEqual("<label>", ImapMessageFlag.GetValidatedKeyword("<label>"));
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("\\Recent", ImapMessageFlag.Recent.ToString());
      Assert.AreEqual("$label", new ImapMessageFlag("$label").ToString());
    }

    [Test]
    public void TestSerializeBinary()
    {
      TestUtils.Assert.IsSerializableBinaryFormat(ImapMessageFlag.Recent, deserialized => {
        Assert.AreNotSame(ImapMessageFlag.Recent, deserialized);
        Assert.AreEqual(ImapMessageFlag.Recent, deserialized);
      });
    }
  }
}
