using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapSearchResultOptionsTests : ImapCombinableDataItemsTestsBase {
    [Test]
    public void TestCombine()
    {
      var o1 = ImapSearchResultOptions.Min;
      var o2 = ImapSearchResultOptions.Max;

      Assert.AreEqual("(MIN MAX)", Send(ImapSearchResultOptions.Combine(o1, o2)));
      Assert.AreEqual("(MIN MAX)", Send(o1 + o2));
      Assert.AreEqual("(MIN MAX)", Send(o1.CombineWith(o2)));

      Assert.AreEqual("(MAX MIN)", Send(ImapSearchResultOptions.Combine(o2, o1)));
      Assert.AreEqual("(MAX MIN)", Send(o2 + o1));
      Assert.AreEqual("(MAX MIN)", Send(o2.CombineWith(o1)));
    }

    [Test]
    public void TestCombine_ContainsRequiredCapability()
    {
      var o1 = ImapSearchResultOptions.All;
      var o2 = ImapSearchResultOptions.Save;
      var capabilitiesESortESearch = new ImapPseudoCapabilitySet(ImapCapability.ESort, ImapCapability.ESearch);
      var capabilitiesESortESearchSearchres = new ImapPseudoCapabilitySet(ImapCapability.ESort, ImapCapability.ESearch, ImapCapability.Searchres);

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(o1));
      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(o2));
      Assert.IsTrue(capabilitiesESortESearch.IsCapable(o1));
      Assert.IsFalse(capabilitiesESortESearch.IsCapable(o2));
      Assert.IsTrue(capabilitiesESortESearchSearchres.IsCapable(o1));
      Assert.IsTrue(capabilitiesESortESearchSearchres.IsCapable(o2));

      foreach (var combined in new[] {
        ImapSearchResultOptions.Combine(o1, o2),
        o1 + o2,
        o1.CombineWith(o2),
      }) {
        Assert.AreEqual("(ALL SAVE)", Send(combined));

        Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(combined));
        Assert.IsFalse(capabilitiesESortESearch.IsCapable(combined));
        Assert.IsTrue(capabilitiesESortESearchSearchres.IsCapable(combined));
      }
    }

    [Test]
    public void TestSave()
    {
      var options = ImapSearchResultOptions.Save;

      Assert.AreEqual("(SAVE)", Send(options));

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(options));
      Assert.IsFalse(new ImapPseudoCapabilitySet(ImapCapability.ESearch).IsCapable(options));
      Assert.IsFalse(new ImapPseudoCapabilitySet(ImapCapability.ESort).IsCapable(options));
      Assert.IsFalse(new ImapPseudoCapabilitySet(ImapCapability.Searchres).IsCapable(options));
      Assert.IsFalse(new ImapPseudoCapabilitySet(ImapCapability.ESearch, ImapCapability.ESort).IsCapable(options));
      Assert.IsFalse(new ImapPseudoCapabilitySet(ImapCapability.ESearch, ImapCapability.Searchres).IsCapable(options));
      Assert.IsFalse(new ImapPseudoCapabilitySet(ImapCapability.ESort, ImapCapability.Searchres).IsCapable(options));
      Assert.IsTrue(new ImapPseudoCapabilitySet(ImapCapability.ESearch, ImapCapability.ESort, ImapCapability.Searchres).IsCapable(options));
    }
  }
}
