using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapSortCriteriaTests : ImapCombinableDataItemsTestsBase {
    [Test]
    public void TestCombine()
    {
      var c1 = ImapSortCriteria.Size;
      var c2 = ImapSortCriteria.Arrival;

      Assert.AreEqual("(SIZE ARRIVAL)", Send(ImapSortCriteria.Combine(c1, c2)));
      Assert.AreEqual("(SIZE ARRIVAL)", Send(c1 + c2));
      Assert.AreEqual("(SIZE ARRIVAL)", Send(c1.CombineWith(c2)));

      Assert.AreEqual("(ARRIVAL SIZE)", Send(ImapSortCriteria.Combine(c2, c1)));
      Assert.AreEqual("(ARRIVAL SIZE)", Send(c2 + c1));
      Assert.AreEqual("(ARRIVAL SIZE)", Send(c2.CombineWith(c1)));
    }

    [Test]
    public void TestCombine_ContainsRequiredCapability_1()
    {
      var c1 = ImapSortCriteria.Date;
      var c2 = ImapSortCriteria.Subject;
      var capabilitiesSort = new ImapPseudoCapabilitySet(ImapCapability.Sort);

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(c1));
      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(c2));
      Assert.IsTrue(capabilitiesSort.IsCapable(c1));
      Assert.IsTrue(capabilitiesSort.IsCapable(c2));

      foreach (var combined in new[] {
        ImapSortCriteria.Combine(c1, c2),
        c1 + c2,
        c1.CombineWith(c2),
      }) {
        Assert.AreEqual("(DATE SUBJECT)", Send(combined));

        Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(combined));
        Assert.IsTrue(capabilitiesSort.IsCapable(combined));
      }
    }

    [Test]
    public void TestCombine_ContainsRequiredCapability_2()
    {
      var c1 = ImapSortCriteria.Date;
      var c2 = ImapSortCriteria.DisplayFrom;
      var capabilitiesSort = new ImapPseudoCapabilitySet(ImapCapability.Sort);
      var capabilitiesSortDisplay = new ImapPseudoCapabilitySet(ImapCapability.Sort, ImapCapability.SortDisplay);

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(c1));
      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(c2));
      Assert.IsTrue(capabilitiesSort.IsCapable(c1));
      Assert.IsFalse(capabilitiesSort.IsCapable(c2));
      Assert.IsTrue(capabilitiesSortDisplay.IsCapable(c1));
      Assert.IsTrue(capabilitiesSortDisplay.IsCapable(c2));

      foreach (var combined in new[] {
        ImapSortCriteria.Combine(c1, c2),
        c1 + c2,
        c1.CombineWith(c2),
      }) {
        Assert.AreEqual("(DATE DISPLAYFROM)", Send(combined));

        Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(combined));
        Assert.IsFalse(capabilitiesSort.IsCapable(combined));
        Assert.IsTrue(capabilitiesSortDisplay.IsCapable(combined));
      }
    }
  }
}
