using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapStatusDataItemsTests : ImapCombinableDataItemsTestsBase {
    [Test]
    public void TestCombine()
    {
      var status1 = ImapStatusDataItems.Recent;
      var status2 = ImapStatusDataItems.UidNext;

      Assert.AreEqual("(RECENT UIDNEXT)", Send(ImapStatusDataItems.Combine(status1, status2)));
      Assert.AreEqual("(RECENT UIDNEXT)", Send(status1 + status2));
      Assert.AreEqual("(RECENT UIDNEXT)", Send(status1.CombineWith(status2)));

      Assert.AreEqual("(UIDNEXT RECENT)", Send(ImapStatusDataItems.Combine(status2, status1)));
      Assert.AreEqual("(UIDNEXT RECENT)", Send(status2 + status1));
      Assert.AreEqual("(UIDNEXT RECENT)", Send(status2.CombineWith(status1)));
    }

    [Test]
    public void TestCombine_ContainsRequiredCapability()
    {
      var status1 = ImapStatusDataItems.HighestModSeq;
      var status2 = ImapStatusDataItems.UidValidity;
      var capabilitiesCondStore = new ImapPseudoCapabilitySet(ImapCapability.CondStore);

      Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(status1));
      Assert.IsTrue(ImapPseudoCapabilitySet.Empty.IsCapable(status2));
      Assert.IsTrue(capabilitiesCondStore.IsCapable(status1));
      Assert.IsTrue(capabilitiesCondStore.IsCapable(status2));

      foreach (var combined in new[] {
        ImapStatusDataItems.Combine(status1, status2),
        status1 + status2,
        status1.CombineWith(status2),
      }) {
        Assert.AreEqual("(HIGHESTMODSEQ UIDVALIDITY)", Send(combined));

        Assert.IsFalse(ImapPseudoCapabilitySet.Empty.IsCapable(combined));
        Assert.IsTrue(capabilitiesCondStore.IsCapable(combined));
      }
    }
  }
}
