using System;
using NUnit.Framework;

using Smdn.Net.Imap4.Protocol.Strings;

namespace Smdn.Net.Imap4.Protocol {
  [TestFixture]
  public class ImapThreadingAlgorithmTests {
    [Test]
    public void TestEquals()
    {
      var x = ImapThreadingAlgorithm.OrderedSubject;

      Assert.IsTrue(x.Equals(ImapThreadingAlgorithm.OrderedSubject));
      Assert.IsTrue(x.Equals((object)"ORDEREDSUBJECT"));
      Assert.IsTrue(x.Equals((object)"orderedsubject"));
      Assert.IsTrue(x.Equals((object)ImapThreadingAlgorithm.OrderedSubject));
      Assert.IsTrue(x.Equals("ORDEREDSUBJECT"));
      Assert.IsTrue(x.Equals("orderedsubject"));
      Assert.IsTrue(x.Equals(ImapThreadingAlgorithm.OrderedSubject.ToString()));

      Assert.IsFalse(x.Equals(ImapThreadingAlgorithm.References));
      Assert.IsFalse(x.Equals((ImapString)ImapThreadingAlgorithm.OrderedSubject.Value));
      Assert.IsFalse(x.Equals(new ImapCapability("ORDEREDSUBJECT")));
      Assert.IsFalse(x.Equals((object)new ImapCapability("ORDEREDSUBJECT")));
    }

    [Test]
    public void TestOpEquality()
    {
      var x = ImapThreadingAlgorithm.OrderedSubject;

      Assert.IsTrue(x == ImapThreadingAlgorithm.OrderedSubject);
      Assert.IsFalse(x != ImapThreadingAlgorithm.OrderedSubject);

      Assert.IsFalse(x == ImapThreadingAlgorithm.References);
      Assert.IsTrue(x != ImapThreadingAlgorithm.References);
    }

    [Test]
    public void TestToString()
    {
      Assert.AreEqual("ORDEREDSUBJECT", ImapThreadingAlgorithm.OrderedSubject.ToString());
    }

    [Test]
    public void TestSerializeBinary()
    {
      TestUtils.Assert.IsSerializableBinaryFormat(ImapThreadingAlgorithm.OrderedSubject, deserialized => {
        Assert.AreNotSame(ImapThreadingAlgorithm.OrderedSubject, deserialized);
        Assert.AreEqual(ImapThreadingAlgorithm.OrderedSubject, deserialized);
      });
    }
  }
}
