using System;
using NUnit.Framework;

namespace Smdn.Net.MessageAccessProtocols.Strings {
  [TestFixture]
  public class TypedStringUtilsTests {
    private class TypedString : IEquatable<TypedString> {
      public static readonly TypedString Imap4Rev1 = new TypedString("IMAP4rev1");
      public static readonly TypedString UidPlus = new TypedString("UIDPLUS");
      public static readonly TypedString AuthPlain = new TypedString("AUTH=PLAIN");

      private static readonly TypedString PrivateClassValue = new TypedString("PrivateClassValue");
      public TypedString PublicInstanceValue = PrivateClassValue;

      public string Value { get; }

      public TypedString(string val)
      {
        this.Value = val;
      }

      public bool Equals(TypedString other)
      {
        return ReferenceEquals(this, other) || string.Equals(this?.Value, other?.Value, StringComparison.OrdinalIgnoreCase);
      }

      public override string ToString() => Value;
    }

    [Test]
    public void TestGetDefinedConstants()
    {
      var expected = new TypedStringSet<TypedString>(new[] {
        new TypedString("IMAP4rev1"),
        new TypedString("UIDPLUS"),
        new TypedString("AUTH=PLAIN"),
      });
      var actual = new TypedStringSet<TypedString>(TypedStringUtils.GetDefinedConstants<TypedString>());

      CollectionAssert.AreEquivalent(expected, actual);
    }
  }
}
